@@ -9,11 +9,12 @@ import os
 import pickle as pickle
 import traceback
 import types
+import typing
 import zipfile
 from dataclasses import dataclass, field
 from io import StringIO
-from logging import debug
-from typing import Dict
+from logging import debug, info
+from copy import deepcopy
 
 import numpy as np
 
@@ -27,6 +28,19 @@ from .exceptions import FomError, GenxIOError, ModelError, ParameterError
 from .models.lib.parameters import get_parameters, NumericParameter
 from .parameters import Parameters
 
+
+try:
+    from yaml import dumper
+except ImportError:
+    pass
+else:
+    class DumpDict(dict):
+        # Make compact layout for non-orso header items read from files
+        def yaml_representer(self, dumper):
+            return dumper.represent_mapping(dumper.DEFAULT_MAPPING_TAG,
+                                            dict(self), flow_style=True)
+
+
 @dataclass
 class StartupScript(BaseConfig):
     section = 'startup'
@@ -47,55 +61,63 @@ class SolverParameters(BaseConfig):
     ignore_fom_nan: bool = True
     ignore_fom_inf: bool = True
 
-    limit_fit_range:bool=False
-    fit_xmin:float=BaseConfig.GParam(0.0, pmin=-1000., pmax=1000.)
-    fit_xmax:float=BaseConfig.GParam(180.0, pmin=-1000., pmax=1000.)
+    limit_fit_range: bool = False
+    fit_xmin: float = BaseConfig.GParam(0.0, pmin=-1000., pmax=1000.)
+    fit_xmax: float = BaseConfig.GParam(180.0, pmin=-1000., pmax=1000.)
+
+    groups = {
+        'FOM': ['figure_of_merit', ['ignore_fom_nan', 'ignore_fom_inf'],
+                'limit_fit_range', ['fit_xmin', 'fit_xmax']]
+        }
 
-    groups={'FOM': ['figure_of_merit', ['ignore_fom_nan', 'ignore_fom_inf'],
-                    'limit_fit_range', ['fit_xmin', 'fit_xmax']]}
 
 class GenxScriptModule(types.ModuleType):
     data: DataList
+    _sim: bool
 
     def __init__(self, data: DataList):
         types.ModuleType.__init__(self, 'genx_script_module')
-        self.__package__=fom_funcs.__package__
-        self.data=data
-        self._sim=False
+        self.__package__ = fom_funcs.__package__
+        self.data = data
+        self._sim = False
+        self.TextIO = typing.TextIO
 
     @staticmethod
     def Sim(data: DataList):
         # default implementation of Sim function, will be overwritten by user.
         return [di.y*0 for di in data]
 
+
 class Model(H5HintedExport):
     ''' A class that holds the model i.e. the script that defines
         the model and the data + various other attributes.
     '''
-    h5group_name='current'
+    h5group_name = 'current'
+    _group_attr = {'NX_class': 'NXentry', 'default': 'data'}
 
-    saved=True
-    fom=None
-    fom_mask_func=None
+    saved = True
+    fom = None
+    fom_mask_func = None
 
     # parameters stored to file
-    script:str
-    fomfunction:str
-    data:DataList
-    parameters:Parameters
+    script: str
+    fomfunction: str
+    data: DataList
+    parameters: Parameters
+    sequence_value: float = 0.
 
     @property
     def fomfunction(self):
         return self.fom_func.__name__
+
     @fomfunction.setter
     def fomfunction(self, value):
         if value in fom_funcs.func_names:
             self.set_fom_func(eval('fom_funcs.'+value))
-            self.solver_parameters.figure_of_merit=value
+            self.solver_parameters.figure_of_merit = value
         else:
             iprint("Can not find fom function name %s"%value)
 
-
     def __init__(self):
         '''
         Create a instance and init all the variables.
@@ -136,7 +158,7 @@ class Model(H5HintedExport):
         self.set_fom_from_config()
 
     def set_fom_from_config(self):
-        self.fomfunction=self.solver_parameters.figure_of_merit
+        self.fomfunction = self.solver_parameters.figure_of_merit
 
     def load(self, filename):
         ''' 
@@ -210,17 +232,28 @@ class Model(H5HintedExport):
 
     def __getstate__(self):
         # generate a pickleable object for thie model, it cannot contain dynamically generated functions
-        state=self.__dict__.copy()
-        del state['fom_mask_func']
-        del state['script_module']
+        state = self.__dict__.copy()
+        if 'fom_mask_func' in state:
+            del state['fom_mask_func']
+        if 'script_module' in state:
+            del state['script_module']
         return state
 
     def __setstate__(self, state):
         self.__dict__.update(state)
         self.create_fom_mask_func()
-        if self.compiled: # if the model was compiled before pickling, do it now
+        if self.compiled:  # if the model was compiled before pickling, do it now
             self.compile_script()
 
+    def copy(self) -> "Model":
+        state = self.__getstate__().copy()
+        output = Model()
+        output.__setstate__(state)
+        return output
+
+    def deepcopy(self) -> "Model":
+        return deepcopy(self)
+
     def read_h5group(self, group):
         """
         Read the parameters from a hdf5 group
@@ -252,7 +285,7 @@ class Model(H5HintedExport):
             raise GenxIOError(str(e), self.filename)
         savefile.close()
 
-    def load_addition(self, name)->bytes:
+    def load_addition(self, name) -> bytes:
         '''
         Load additional text from sub-file
         '''
@@ -306,6 +339,12 @@ class Model(H5HintedExport):
         result = eval(codestring, self.script_module.__dict__)
         return result
 
+    def set_script_variable(self, name, value):
+        self.script_module.__dict__[name] = value
+
+    def unset_script_variable(self, name):
+        del(self.script_module.__dict__[name])
+
     def create_fom_mask_func(self):
         """
         Create a mask func for fom to take care of unallowed values.
@@ -434,7 +473,7 @@ class Model(H5HintedExport):
             # noinspection PyUnresolvedReferences
             return self.script_module.__tempfunc__
 
-    def get_fit_pars(self):
+    def get_fit_pars(self, use_bounds=True):
         '''
         Returns the parameters used with fitting. i.e. the function to 
         set the paraemters, the guess value (values), minimum allowed values
@@ -443,14 +482,15 @@ class Model(H5HintedExport):
         (row_numbers, sfuncs, vals, minvals, maxvals) = self.parameters.get_fit_pars()
         if len(sfuncs)==0:
             raise ParameterError(sfuncs, 0, None, 4)
-        # Check for min and max on all the values
-        for i in range(len(vals)):
-            # parameter less than min
-            if vals[i]<minvals[i]:
-                raise ParameterError(sfuncs[i], row_numbers[i], None, 3)
-            # parameter larger than max
-            if vals[i]>maxvals[i]:
-                raise ParameterError(sfuncs[i], row_numbers[i], None, 2)
+        if use_bounds:
+            # Check for min and max on all the values
+            for i in range(len(vals)):
+                # parameter less than min
+                if vals[i]<minvals[i]:
+                    raise ParameterError(sfuncs[i], row_numbers[i], None, 3)
+                # parameter larger than max
+                if vals[i]>maxvals[i]:
+                    raise ParameterError(sfuncs[i], row_numbers[i], None, 2)
 
         # Compile the strings to create the functions..
         funcs = []
@@ -541,7 +581,7 @@ class Model(H5HintedExport):
         model_copy.compiled = self.compiled
         model_copy.fom = self.fom
         model_copy.solver_parameters = self.solver_parameters.copy()
-        model_copy.startup_script=self.startup_script.copy()
+        model_copy.startup_script = self.startup_script.copy()
         model_copy.opt = self.opt.copy()
         model_copy.saved = self.saved
         # Needs to reset the fom_mask_func since this fails under windows.
@@ -573,7 +613,7 @@ class Model(H5HintedExport):
         The fileending will be .ort
         '''
         self.simulate(True)
-        from orsopy.fileio import save_orso, OrsoDataset, Orso, Column
+        from orsopy.fileio import save_orso, OrsoDataset, Orso, Column, ErrorColumn
         from .version import __version__ as version
         para_list = [dict([(nj, tpj(pij)) for nj, pij, tpj in zip(self.parameters.data_labels, pi,
                                                                   [str, float, bool, float, float, str])])
@@ -590,7 +630,7 @@ class Model(H5HintedExport):
         ds = []
         for di in self.data:
             header = Orso.empty().to_dict()
-            header.update(di.meta)
+            self.update_dictionary(header, di.meta)
             try:
                 import getpass
                 # noinspection PyTypeChecker
@@ -603,13 +643,21 @@ class Model(H5HintedExport):
             header.update(add_header)
             header['data_set'] = di.name
             columns = [di.x, di.y, di.error]
-            if hasattr(self.script_module, 'inst') and self.script_module.inst.coords in ['tth', '2θ']:
-                column_names = [Column('TTh', 'deg'), Column('R'), Column('sR')]
+            if 'columns' in header:
+                prev_columns_list = [Column(**ci) if 'name' in ci else ErrorColumn(**ci) for ci in header['columns']]
+                prev_columns = dict([(ci.name, ci) for ci in prev_columns_list])
             else:
-                column_names = [Column('Qz', '1/angstrom'), Column('R'), Column('sR')]
+                prev_columns_list = None
+                prev_columns = {}
+            if hasattr(self.script_module, 'inst') and self.script_module.inst.coords in ['tth', '2θ']:
+                # this will probably need changing in the future as ORSO standard
+                # will probably require Qz for first column
+                column_names = [Column('TTh', 'deg'), Column('R'), ErrorColumn('R')]
+            else:
+                column_names = [Column('Qz', '1/angstrom'), Column('R'), ErrorColumn('R')]
             if 'res' in di.extra_data:
                 columns.append(di.extra_data['res'])
-                column_names.append(Column('s'+column_names[0].name, column_names[0].unit))
+                column_names.append(ErrorColumn(column_names[0].name))
             columns.append(di.y_sim)
             column_names.append(Column('Rsim'))
             columns.append(di.y_fom)
@@ -618,14 +666,47 @@ class Model(H5HintedExport):
                 if name=='res':
                     continue
                 columns.append(col)
-                column_names.append(Column(name))
+                if name in prev_columns:
+                    column_names.append(prev_columns[name])
+                else:
+                    column_names.append(Column(name))
             header['columns'] = column_names
-            ds.append(OrsoDataset(Orso(**header), np.array(columns).T))
+            header_obj = Orso(**header)
+            for key, value in header_obj._user_data.items():
+                if not key in ['analysis'] and isinstance(value, dict):
+                    header_obj._user_data[key] = DumpDict(value)
+            ds.append(OrsoDataset(header_obj, np.array(columns).T))
         try:
             save_orso(ds, basename, data_separator='\n')
         except GenxIOError as e:
             raise GenxIOError(e.error_message, e.file)
 
+    @staticmethod
+    def update_orso_meta(datasets: DataList):
+        from orsopy.fileio import Orso
+        for di in datasets:
+            defaults = Orso.empty().to_dict()
+            Model.update_dictionary(defaults, di.meta)
+            di.meta = defaults
+
+    def validate_orso_meta(self):
+        from orsopy.fileio import base
+        base._validate_header_data([di.meta for di in self.data])
+
+    @staticmethod
+    def update_dictionary(to_update, to_include):
+        """
+        Updates the entries in one dictrionary recursively to keep existing sub-keys.
+        """
+        for key, value in to_include.items():
+            if key in to_update and isinstance(to_update[key], dict):
+                if not isinstance(value, dict):
+                    info(f"Metadata for {key} according to specification should be dictionary, found {value}")
+                else:
+                    Model.update_dictionary(to_update[key], value)
+            else:
+                to_update[key]=value
+
     def export_data(self, basename):
         '''
         Export the data to files with basename filename. ASCII output. 
@@ -667,50 +748,36 @@ class Model(H5HintedExport):
         Save the string to file with filename.
         '''
         try:
-            savefile = open(filename, 'w')
+            with open(filename, 'w', encoding='utf-8') as savefile:
+                savefile.write(save_string)
         except Exception as e:
             raise GenxIOError(e.__str__(), filename)
 
-        # Save the string to file
-        try:
-            savefile.write(save_string)
-        except Exception as e:
-            raise GenxIOError(e.__str__(), filename)
-
-        savefile.close()
-
     def _read_from_file(self, filename):
         '''
         Reads the entrie file into string and returns it.
         '''
         try:
-            loadfile = open(filename, 'r')
+            with open(filename, 'r', encoding='utf-8') as loadfile:
+                read_string = loadfile.read()
         except Exception as e:
             raise GenxIOError(e.__str__(), filename)
 
-        # Read the text from file
-        try:
-            read_string = loadfile.read()
-        except Exception as e:
-            raise GenxIOError(e.__str__(), filename)
-
-        loadfile.close()
-
         return read_string
 
-    def get_parameters(self)->Parameters:
+    def get_parameters(self) -> Parameters:
         return self.parameters
 
-    def get_data(self)->DataList:
+    def get_data(self) -> DataList:
         return self.data
 
-    def get_script(self)->str:
+    def get_script(self) -> str:
         return self.script
 
-    def get_filename(self)->str:
+    def get_filename(self) -> str:
         return self.filename
 
-    def get_possible_parameters(self)->dict:
+    def get_possible_parameters(self) -> dict:
         """
         Returns all the parameters that can be fitted. Is used by the parameter grid.
         """
@@ -719,7 +786,7 @@ class Model(H5HintedExport):
             par_dict = self.get_possible_set_functions()
         return par_dict
 
-    def get_possible_set_functions(self)->dict:
+    def get_possible_set_functions(self) -> dict:
         """
         Returns all the parameters that can be fitted given by the old style of defining parameters GenX2.4.X
         """
@@ -782,9 +849,9 @@ class Model(H5HintedExport):
             # Add this to the right item in par_dict given
             # its class and name.
             [par_dict[obj.__class__.__name__].__setitem__(name,
-                                                        [member for member in dir(obj)
-                                                        if member.startswith(self.opt.set_func)])
-                                                        for name, obj in valid_objs]
+                                                          [member for member in dir(obj)
+                                                           if member.startswith(self.opt.set_func)])
+             for name, obj in valid_objs]
             return par_dict
         return {}
 
@@ -795,7 +862,7 @@ class Model(H5HintedExport):
     def set_fom_func(self, fom_func: callable):
         self.fom_func = fom_func
 
-    def is_compiled(self)->bool:
+    def is_compiled(self) -> bool:
         return self.compiled
 
     def bumps_problem(self):
@@ -813,6 +880,17 @@ class Model(H5HintedExport):
 
         return FitProblem(GenxCurve(self))
 
+    def asym_stderr(self, dream_fit):
+        """
+        Approximate standard error as distance to lower and upper bound of
+        the 68% interval for the sample.
+        Based on the stderr estimation from dream fit in bumps.
+        """
+        from bumps.dream.stats import var_stats
+
+        vstats = var_stats(dream_fit.state.draw(portion=dream_fit._trimmed))
+        return np.array([(v.p68[0]-v.best, v.p68[1]-v.best) for v in vstats], 'd')
+
     def bumps_fit(self, method='dream',
                   pop=15, samples=1e5, burn=100, steps=0,
                   thin=1, alpha=0, outliers='none', trim=False,
@@ -833,8 +911,8 @@ class Model(H5HintedExport):
 
         if problem is None:
             problem = self.bumps_problem()
-        problem.fitness.stop_fit=False
-        options['abort_test']=lambda: problem.fitness.stop_fit
+        problem.fitness.stop_fit = False
+        options['abort_test'] = lambda: problem.fitness.stop_fit
 
         # verbose = True
         if method not in FIT_AVAILABLE_IDS:
@@ -850,7 +928,11 @@ class Model(H5HintedExport):
         x, fx = driver.fit()
         problem.setp(x)
         dx = driver.stderr()
-        result = BumpsResult(x=x, dx=dx, cov=driver.cov(), bproblem=problem)
+        if method=='dream':
+            dxpm = self.asym_stderr(driver.fitter)
+        else:
+            dxpm = None
+        result = BumpsResult(x=x, dx=dx, dxpm=dxpm, cov=driver.cov(), chisq=driver.chisq(), bproblem=problem)
         if hasattr(driver.fitter, 'state'):
             result.state = driver.fitter.state
         return result
@@ -930,6 +1012,10 @@ class Model(H5HintedExport):
         self.simulate()
         return self.data.plot(data_labels=data_labels, sim_labels=sim_labels)
 
+    def __eq__(self, other: "Model"):
+        # compare relevant parts of model object agains another
+        return self.data==other.data and self.script==other.script and self.parameters==other.parameters
+
     def __repr__(self):
         """
         Display information about the model.
@@ -1042,6 +1128,7 @@ class Model(H5HintedExport):
     def _ipyw_script(self, change):
         self.set_script(change.new)
 
+
 class GenxCurve:
     """
     Bumps Curve object for a GenX model.
@@ -1058,7 +1145,7 @@ class GenxCurve:
 
         if not self.model.compiled:
             self.model.compile_script()
-        self.model_script=self.model.script_module
+        self.model_script = self.model.script_module
 
         self.name = "GenX model"
         self.plot_x = None
@@ -1066,12 +1153,12 @@ class GenxCurve:
         pars, state, funcs = self._parse_pars()
 
         self._pnames = list(pars.keys())
-        self._pars=pars
+        self._pars = pars
         self._state = state
-        self._set_funcs=funcs
+        self._set_funcs = funcs
         self._cached_theory = None
-        self.stop_fit=False
-        self.n_fev=0
+        self.stop_fit = False
+        self.n_fev = 0
 
     @property
     def x(self):
@@ -1087,23 +1174,23 @@ class GenxCurve:
 
     def _parse_pars(self):
         from bumps.parameter import Parameter
-        pars={}
-        state={}
-        funcs={}
+        pars = {}
+        state = {}
+        funcs = {}
         for p in self.model.parameters:
             if p.name.strip()=='':
                 continue
             name = p.name.replace('.set', '_')
             if p.fit:
-                pars[name]=Parameter.default(p.value, name=name, bounds=(p.min, p.max))
+                pars[name] = Parameter.default(p.value, name=name, bounds=(p.min, p.max))
             else:
-                state[name]=p.value
-            funcs[name]=self.model.create_fit_func(p.name)
+                state[name] = p.value
+            funcs[name] = self.model.create_fit_func(p.name)
         return pars, state, funcs
 
     def update(self):
         self._cached_theory = None
-        self.n_fev=0
+        self.n_fev = 0
 
     def parameters(self):
         return self._pars
@@ -1121,8 +1208,8 @@ class GenxCurve:
 
     def _compute_theory(self, x):
         self._apply_par(x)
-        sim=self.model_script.Sim(self.model.data)
-        self.n_fev+=1
+        sim = self.model_script.Sim(self.model.data)
+        self.n_fev += 1
         return np.hstack([si for si, di in zip(sim, self.model.data) if di.use])
 
     def _apply_par(self, x):
@@ -1134,19 +1221,19 @@ class GenxCurve:
     def simulate_data(self, noise=None):
         theory = self.theory()
         if noise is not None:
-            if noise == 'data':
+            if noise=='data':
                 pass
-            elif noise < 0:
+            elif noise<0:
                 self.dy = -0.01*noise*theory
             else:
                 self.dy = noise
-        self.y = theory + np.random.randn(*theory.shape)*self.dy
+        self.y = theory+np.random.randn(*theory.shape)*self.dy
 
     def residuals(self):
         return (self.theory()-self.y)/self.dy
 
     def nllf(self):
-        r=self.residuals()
+        r = self.residuals()
         fom = np.sum(r**2)
         penalty_funcs = self.model.get_par_penalty()
         if len(penalty_funcs)>0 and fom is not np.NAN:
@@ -1154,7 +1241,7 @@ class GenxCurve:
         return 0.5*fom
 
     def __getstate__(self):
-        state=self.__dict__.copy()
+        state = self.__dict__.copy()
         del state['model_script']
         del state['_set_funcs']
         return state
@@ -1163,10 +1250,10 @@ class GenxCurve:
         self.__dict__.update(state)
         if not self.model.compiled:
             self.model.compile_script()
-        self.model_script=self.model.script_module
+        self.model_script = self.model.script_module
 
         pars, state, funcs = self._parse_pars()
         self._pnames = list(pars.keys())
-        self._pars=pars
+        self._pars = pars
         self._state = state
-        self._set_funcs=funcs
+        self._set_funcs = funcs
