@@ -12,12 +12,15 @@ Last Changes 04/22/15
 '''
 
 import os, sys
-from copy import deepcopy
 import wx
+
+from copy import deepcopy
+from wx.lib.mixins.listctrl import ListCtrlAutoWidthMixin
+
 from .. import add_on_framework as framework
+from ..utils import ShowInfoDialog, ShowQuestionDialog, ShowWarningDialog
 from genx.parameters import Parameters
 import genx.gui.images as img
-from wx.lib.mixins.listctrl import ListCtrlAutoWidthMixin
 
 class Plugin(framework.Template):
     _refplugin=None
@@ -69,11 +72,7 @@ class Plugin(framework.Template):
             # connect to the reflectivity plugin for layer creation
             self._refplugin=ph.loaded_plugins['Reflectivity']
         else:
-            dlg=wx.MessageDialog(self.materials_panel, 'Reflectivity plugin must be loaded',
-                                 caption='Information',
-                                 style=wx.OK | wx.ICON_WARNING)
-            dlg.ShowModal()
-            dlg.Destroy()
+            ShowWarningDialog(self.materials_panel, 'Reflectivity plugin must be loaded', 'Information')
             self._refplugin=None
 
     def create_toolbar(self):
@@ -176,18 +175,19 @@ class Plugin(framework.Template):
                         plot.ax.plot(di.x, di.y_sim, c=di.sim_color,
                                      lw=di.sim_linethickness, ls=styles[k%len(styles)],
                                      marker=di.sim_symbol, ms=di.sim_symbolsize)
-                        for key in slds[i]:
-                            is_imag=key[:2]=='Im' or key[:4]=='imag'
-                            if (is_imag and self.refplugin.show_imag_sld) or not is_imag:
-                                if key!='z' and key!='SLD unit':
-                                    sld_plot.plot.ax.plot(slds[i]['z'], slds[i][key],
-                                                          colors[j%len(colors)], ls=styles[k%len(styles)],
-                                                          label=None)
-                                    j+=1
+                        # if len(slds)>i:
+                        #     for key in slds[i]:
+                        #         is_imag=key[:2]=='Im' or key[:4]=='imag'
+                        #         if (is_imag and self.refplugin.show_imag_sld) or not is_imag:
+                        #             if key!='z' and key!='SLD unit':
+                        #                 sld_plot.ax.plot(slds[i]['z'], slds[i][key],
+                        #                                       colors[j%len(colors)], ls=styles[k%len(styles)],
+                        #                                       label=None)
+                        #                 j+=1
                 k+=1
         plot.AutoScale()
         plot.flush_plot()
-        sld_plot.plot.flush_plot()
+        #sld_plot.flush_plot()
 
     def OnRecalcAll(self, event):
         pl=self.parameter_list
@@ -333,11 +333,7 @@ class ParameterList(wx.ListCtrl, ListCtrlAutoWidthMixin):
         '''
         # Check so that one dataset is selected
         if len(indices)==0:
-            dlg=wx.MessageDialog(self,
-                                 'At least one data set has to be selected'
-                                 , caption='Information', style=wx.OK | wx.ICON_INFORMATION)
-            dlg.ShowModal()
-            dlg.Destroy()
+            ShowInfoDialog(self, 'At least one data set has to be selected')
             return False
         return True
 
@@ -369,9 +365,8 @@ class ParameterList(wx.ListCtrl, ListCtrlAutoWidthMixin):
         if not was_stored:
             message='Do you want to store the current parameters in the Vault'+ \
                     ' before applying the selected ones?'
-            dlg=wx.MessageDialog(self, message, 'Store current parameters?',
-                                 wx.YES_NO | wx.ICON_QUESTION)
-            if dlg.ShowModal()==wx.ID_YES:
+            result=ShowQuestionDialog(self, message, 'Store current parameters?')
+            if result:
                 self.AddItem(None)
         model.parameters.data=deepcopy(self.parameter_list[index][2])
         self.plugin.parent.paramter_grid._grid_changed()
