@@ -16,6 +16,7 @@ import io, traceback
 import wx.grid as gridlib
 from wx.adv import Wizard, WizardPageSimple
 from orsopy.slddb import api
+from orsopy.slddb.material import Formula as MatFormula
 
 from .. import add_on_framework as framework
 from genx.model import Model
@@ -48,12 +49,13 @@ except:
     api = None
 
 
-def get_mat_api(frm):
+def get_mat_api(frm: Formula):
     # Use ORSO SLD db to query for a material by formula
     if not api:
         return None
     try:
-        res = api.localquery(dict(formula=frm.estr()))
+        frm2 = MatFormula(frm.estr())
+        res = api.localquery(dict(formula=str(frm2)))
         if len(res):
             mat = api.localmaterial(res[0]['ID'])
             return mat.dens, res[0]['ID'], res[0]['validated']
@@ -197,8 +199,8 @@ class SampleTable(gridlib.GridTableBase):
 
         dpi_scale_factor = wx.GetApp().dpi_scale_factor
 
-        self.grid.SetRowLabelSize(30*dpi_scale_factor)
-        self.grid.SetColLabelSize(50*dpi_scale_factor)
+        self.grid.SetRowLabelSize(int(30*dpi_scale_factor))
+        self.grid.SetColLabelSize(int(50*dpi_scale_factor))
         for i, colinfo in enumerate(self._columns):
             # self.parent.SetColSize(i, 50)
             self.grid.AutoSizeColumn(i, True)
@@ -881,7 +883,7 @@ class SamplePanel(wx.Panel):
 
         newid = wx.NewId()
         button = wx.Button(self.toolbar, newid, label='Instrument Settings',
-                           size=(132*dpi_scale_factor, 22*dpi_scale_factor))
+                           size=(int(132*dpi_scale_factor), int(22*dpi_scale_factor)))
         button.SetBitmap(wx.Bitmap(images.instrument.GetImage().Scale(tb_bmp_size, tb_bmp_size)), dir=wx.LEFT)
         self.toolbar.AddControl(button)
         self.Bind(wx.EVT_BUTTON, self.EditInstrument, id=newid)
@@ -891,7 +893,7 @@ class SamplePanel(wx.Panel):
 
         newid = wx.NewId()
         button = wx.Button(self.toolbar, newid, label='to Advanced Modelling',
-                           size=(150*dpi_scale_factor, 22*dpi_scale_factor))
+                           size=(int(150*dpi_scale_factor), int(22*dpi_scale_factor)))
         button.SetBitmap(wx.Bitmap(images.custom_parameters.GetImage().Scale(tb_bmp_size, tb_bmp_size)), dir=wx.LEFT)
         button.SetToolTip("Switch to Reflectivity plugin for advanced modeling options.\n"
                           "This converts the model and can't be undone.")
@@ -967,25 +969,25 @@ class SamplePanel(wx.Panel):
             di.run_command()
             script += "    # BEGIN Dataset %i DO NOT CHANGE\n" \
                       "    d = data[%i]\n"%(i, i)
-            if hasattr(di, 'res') and di.res is not None:
-                script += "    inst.setRes(data[%i].res)\n"%i
-            elif res_set:
-                script += "    inst.setRes(0.001)\n"
             inst_id = i%len(insts)
             try:
                 # extract polarization channel from ORSO metadata
                 pol = di.meta['data_source']['measurement']['instrument_settings']['polarization']
                 if pol=='unpolarized':
                     pass
-                elif pol in ['m', 'mm']:
+                elif pol in ['mo', 'om', 'mm']:
                     inst_id = 1%len(insts)
                 else:
                     inst_id = 0
             except KeyError:
                 pass
+            if hasattr(di, 'res') and di.res is not None:
+                script += "    %s.setRes(data[%i].res)\n"%(insts[inst_id], i)
+            elif res_set:
+                script += "    %s.setRes(0.001)\n"%(insts[inst_id])
             script += "    I.append(sample.SimSpecular(d.x, %s))\n" \
-                      "    if _sim: SLD.append(sample.SimSLD(None, None, inst))\n" \
-                      "    # END Dataset %i\n"%(insts[inst_id], i)
+                      "    if _sim: SLD.append(sample.SimSLD(None, None, %s))\n" \
+                      "    # END Dataset %i\n"%(insts[inst_id], insts[inst_id], i)
             if re_color and len(insts)>1:
                 if inst_id==0:
                     di.data_color = (0.7, 0.0, 0.0)
@@ -1180,10 +1182,10 @@ class WizarSelectionPage(WizardPageSimple):
         out_layout = wx.BoxSizer(wx.VERTICAL)
         box.SetSizer(out_layout)
         box_layout = wx.GridSizer(min(4, len(choices)//4+1), 2, 2)
-        out_layout.Add(box_layout, 1, wx.EXPAND | wx.TOP | wx.LEFT, 12*dpi_scale_factor)
+        out_layout.Add(box_layout, 1, wx.EXPAND | wx.TOP | wx.LEFT, int(12*dpi_scale_factor))
         for choice in choices:
             self.ctrl[choice] = wx.RadioButton(box, label=choice,
-                                               size=wx.Size(-1, 16*dpi_scale_factor))
+                                               size=wx.Size(-1, int(16*dpi_scale_factor)))
             box_layout.Add(self.ctrl[choice], 0, wx.FIXED_MINSIZE, 2)
         self.ctrl[choices[0]].SetValue(True)
 
